function [hol,found,precise,timee]=FAST(hologram,mrate,pix,lambda,ranges,FOV,REMOVED_I0,Mask,Alpha_thresh,factors,K,q,itter,SecondT,UNIF_Z,A0,sigma,MEXQmap,MEXmodelFAST,NoFigure)
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Jan 2012
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Goal : process of a Digital Hologram to extract the 3D coordinates and
% the Radius of spherical particle using Inverse Approach
%
%-------------------------------------------------------------------------%
% inputs:         
% hologram
%
% mrate         = the rate of the searching widnow comparing to the
%                hologram's size - no unit
%
% pix           = pixel size in meter
%
% lambda        = wavelenght of the laser with which the hologram has
%               been recorded
%
% ranges        = ranges for Radius and Z distance of particles,
%               known from the experiment and the setup
%
% FOV           = 0 if one searches out of the filed of view od sensor for particles
% I0            = if you have removed the mean of hologram please provide
%               the value of mean. Otherwise pasthis parameter as 0  - I0
%               couldn't be negative!
%   
% Mask          = wieghting mask over considered pixels of sensor

%-------------------------------------------------------------------------%
%outputs        
% found         = coordinates of found particles during global detection 
% precise       = coordinates of all the particles which have been
%               detected from the hologram after local optimization - final values
%
% times_global  = time costs of global detection ffor each
%               particle
%
% times_local   = time costs of local optimization for each
%               particle

%-------------------------------------------------------------------------%

% To test:
% lambda=0.532e-6;
% pix=7e-6; K=sqrt(0.7);
% hol_size=[256 256];
% REMOVED_I0= 0;
% I0=100; A0=0;
% SNR=1000;
% ranges=[0.1 0.3;20e-6 50e-6];
% mrate=[2 2];
% n=5 %number of particles in the hologram
% FOV=1; Mask=0;
% [hologram,particle,REMOVED_I0,noises]=simulate_holo_mex(n,lambda,pix,hol_size(1),hol_size(2),I0,SNR,ranges(1,:),ranges(2,:),1,0,0,K);
% sigma=I0*pi*(max(particle(:,4)).^2)/lambda/min(particle(:,3))/SNR; 
% factors=[0.002 2e-6];
% Mask = 0; itter=20; SecondT=1; UNIF_Z=1; coef=10;
% MEXQmap=0;MEXmodelFAST=0;NoFigure=1;
% Alpha_thresh=0;q=10;
% [hol,found,precise,timee]=FAST(hologram,mrate,pix,lambda,ranges,FOV,REMOVED_I0,Mask,Alpha_thresh,factors,K,q,itter,SecondT,UNIF_Z,A0,sigma,MEXQmap,MEXmodelFAST,NoFigure);

%-------------------------------------------------------------------------%
fprintf(' - This process can take some minutes per particle \n depending on the size of your search space.\n \n - It took us 4 minuets per particle for a volume of \n 7 by 7 by 100 millimeters searching for particles \n with radii between 50 to 70 micrometers.\n \n - Until the process is complete, be our guest and take a Coffee break!\n')


%% Zero-padding

holsize=size(hologram);
pad_size(1)=(mrate(1)+1)*holsize(1);
pad_size(2)=(mrate(2)+1)*holsize(2);
%% calculation of map indices 
% % Filed of search - indexing according to Matlab

    map_FOV1=holsize(1)/2+1:pad_size(1)-holsize(1)/2;
    map_FOV2=holsize(2)/2+1:pad_size(2)-holsize(2)/2;


if (MEXQmap==1)
% % to use mex function of matching pursuit - indexing according to c language
    MFOVX(1)=map_FOV1(1)-1;
    MFOVX(2)=map_FOV1(end);
    MFOVY(1)=map_FOV2(1)-1;
    MFOVY(2)=map_FOV2(end);

end



%% calculation of Field-of-view indices
FOV_size1=(pad_size(1)-holsize(1))/2+1 :(pad_size(1)+holsize(1))/2 ;
FOV_size2=(pad_size(2)-holsize(2))/2+1 :(pad_size(2)+holsize(2))/2 ;

%% initialization
hol=zeros(pad_size(1),pad_size(2));
hol(FOV_size1,FOV_size2)=hologram;

%% finding the best fft algorithm
    fftw('wisdom', []);
    fftw('planner', 'exhaustive');
    dummy=fft2(hol);
    clear dummy

    
    if (exist('Mask','var') && (size(Mask,1)==size(hologram,1)) && (size(Mask,2)==size(hologram,2)))
        if (MEXQmap==1)
            w=zeros(pad_size(1),pad_size(2));
        else
             w=logical(zeros(pad_size(1),pad_size(2)));
        end
        w(FOV_size1,FOV_size2)=Mask;
        ind=find(Mask~=0);


        if REMOVED_I0==0
            I0=mean(mean(hologram(ind)));
            hol1(ind)=hologram(ind)-I0;
        else
            I0=REMOVED_I0;
            hol1=hologram;
        end

        if (NoFigure==0)
        figure;imshow(hol1.*Mask,[]);title('after removing mean using Mask')
        end
    else
        fprintf('- there is no Mask or the size of Mask is not equal to the field of view of the sensor, Mask is set to ones all over the FOV of sensor\n')
        if (MEXQmap==1)
            w=zeros(pad_size(1),pad_size(2));
        else
             w=logical(zeros(pad_size(1),pad_size(2)));
        end
        w(FOV_size1,FOV_size2)=1;

        if REMOVED_I0==0
            I0=mean(mean(hologram));
            hol1=hologram-I0;
        else 
            I0=REMOVED_I0;
            hol1=hologram;
        end

    end

    hol(FOV_size1,FOV_size2)=hol1;

    iI=1;
    flag_glob=1;

    MS=size(hologram);
    MR=mrate;


while(flag_glob>0)% out of particles
    
    N=min(floor(2*sqrt(min(holsize)/10)-2*ranges(2,2)/pix),floor(sqrt(lambda*ranges(1,1)/(2*q+1/2))/pix));% for minimum R
   fprintf('- Down-sampling period in one dimension is %g\n',N)
    flag_loc=1;
    
    while(flag_loc>0) % changing the level of the pyramid
        tic

        if N==1 
            tt=K;% for AccuracyMaps
            hologram_ds=hol;
            mm=sum(sum(hol(FOV_size1,FOV_size2).*w(FOV_size1,FOV_size2)))/sum(sum(w(FOV_size1,FOV_size2)));
%             Ids=mm;
            hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2)-mm;
             wds=w;
               
                 map_FOV1N=map_FOV1;
                 map_FOV2N=map_FOV2;
                 FOV_size1N=FOV_size1;
                 FOV_size2N=FOV_size2;
             if (MEXQmap==1)
                MFOVXN(1)=map_FOV1N(1)-1;
                MFOVXN(2)=map_FOV1N(end);
                MFOVYN(1)=map_FOV2N(1)-1;
                MFOVYN(2)=map_FOV2N(end);
             end
        else
            tt=N;% for AccuracyMaps
            
            fil=zeros(pad_size(1),pad_size(2));
            fil(floor(pad_size(1)/2-N/2)+1:floor(pad_size(1)/2+N/2),floor(pad_size(2)/2-N/2)+1:floor(pad_size(2)/2+N/2))=1/N/N*ones(N);
            hologram_ds=fftshift(ifft2(fft2(hol).*fft2(fil)));   

            % % Down sampling
            hologram_ds=hologram_ds(1:N:end,1:N:end);
            hologram_ds=hologram_ds(1:2*floor(size(hologram_ds,1)/2),1:2*floor(size(hologram_ds,2)/2));%figure;imshow(hologram_ds,[]);colorbar
    %         size(hologram_ds)
            wds=w(1:N:end,1:N:end);
            wds=wds(1:2*floor(size(hologram_ds,1)/2),1:2*floor(size(hologram_ds,2)/2));
    %         figure;imshow(wds,[]);



            map_FOV1N=floor(size(hologram_ds,1)/(mrate(1)+1)/2)+1:floor(size(hologram_ds,1)-size(hologram_ds,1)/(mrate(1)+1)/2);
            map_FOV2N=floor(size(hologram_ds,2)/(mrate(2)+1)/2)+1:floor(size(hologram_ds,2)-size(hologram_ds,2)/(mrate(2)+1)/2);
             if (MEXQmap==1)
                MFOVXN(1)=map_FOV1N(1)-1;
                MFOVXN(2)=map_FOV1N(end);
                MFOVYN(1)=map_FOV2N(1)-1;
                MFOVYN(2)=map_FOV2N(end);
             end
            
            FOV_size1N=floor(size(hologram_ds,1)/2)-floor(size(hologram_ds,1)/2/(mrate(1)+1))+1:floor(size(hologram_ds,1)/2)+floor(size(hologram_ds,1)/2/(mrate(1)+1));
            FOV_size2N=floor(size(hologram_ds,2)/2)-floor(size(hologram_ds,2)/2/(mrate(2)+1))+1:floor(size(hologram_ds,2)/2)+floor(size(hologram_ds,2)/2/(mrate(2)+1));
            
           
            Ids=sum(sum(hologram_ds.*wds))/sum(sum(wds(FOV_size1N,FOV_size2N)));
            hologram_ds(FOV_size1N,FOV_size2N)=hologram_ds(FOV_size1N,FOV_size2N)-Ids;  
        end
        if (NoFigure==0)
        figure;imshow(hologram_ds,[]);title('first DS')
        end
        %% global detection
        fprintf('- Matching Pursuit procedure\n') 
        
        fprintf('- Field of View is %g\n',FOV)
        
        fprintf('=1 to search in the Field of view \n =0 to search out of the Field of view\n')
 
            facts=factors(1:2);
%             facts(1)=facts(1)*2;
            
            tic
            if (MEXQmap==1)
%             % matching pursuit using .mex file for Qmap
                founds=matching_pursuit(hologram_ds,lambda,pix,floor(size(hologram_ds)./(mrate+1)),FOV,wds,ranges,MFOVXN,MFOVYN,N,tt,facts,UNIF_Z,MEXQmap,MEXmodelFAST);
            else
%             % matching pursuit using .m file
                founds=matching_pursuit(hologram_ds,lambda,pix,floor(size(hologram_ds)./(mrate+1)),FOV,wds,ranges,map_FOV1N,map_FOV2N,N,tt,facts,UNIF_Z,MEXQmap,MEXmodelFAST);  
            end

          founds(1:2)=founds(1:2)*pix*N;
          Level=N;


           %% check if we are out of particles

             if (MEXmodelFAST==1)         
                g_current=model_calc_faster(founds(1),founds(2),founds(3),founds(4),pix,lambda,size(hologram_ds),0,tt); 
             else
                 g_current=model_mex(founds(1),founds(2),founds(3),founds(4),pix,lambda,size(hologram_ds),0,tt); 
             end
%             figure;imshow(g_current,[]);title('model after global detection');
            wg=sum(sum(wds.*g_current)); % wg
            wrg=sum(sum(wds.*hologram_ds.*g_current));%wrg
            ws=sum(sum(wds));
            wg2=sum(sum(wds.*(g_current.^2)));%wg2
            
            wr=sum(sum(wds.*hologram_ds));

                alpha1=-ws.*wrg/(ws.*wg2 - wg.^2);%ws.*(wg.*wr-wrg)/(ws.*wg2 - wg.^2);
                I_p1=(wg.^2.*wr-wg.*wrg)./(ws.*wg2 - wg.^2);

            nom=I_p1+I0+Ids;%+abs(REMOVED_I0);
            if N==1
            fprintf('- Value of alpha which should be thresholded is %g\n',alpha1/nom) % value of alpha

            end
            
        
            if ((alpha1/nom < Alpha_thresh) && (FOV==1) )
                 
                 if floor(N/2)>1
                     
                         N=floor(N/2);
                         fprintf('- Down-sampling period in one dimension is %g\n',N)

                         continue;
                 elseif N>1
                     
                     N=1;
                     fprintf('- Down-sampling period in one dimension is %g\n',N)

                     continue;
                     

                 elseif (N==1 && mrate(1)==1 && mrate(2)==1) 
                     if (iI==1)
                        fprintf('- please check :\n 1. your provided range for Z and R \n 2. threshold for alpha \n 3. Can you see any particles on hologram? Remove the background from Raw holograms using a sequence of captured holograms\n')
                     found=zeros(1,4);precise=zeros(1,8);timee=[0 0];
                     end
                        FOV=1;
                        fprintf('- Field of View is %g\n',FOV)
        
                        fprintf('=1 to search in the Field of view \n =0 to search out of the Field of view\n')

                        flag_glob=-1;                  
                        break;

                 else   
                     fprintf('- Searching out of the filed of view using one-scale approach\n')

                     factors(1:2)=factors(1:2)/2;
                         FOV=0;
                         itter=30;
                       [hol,found,precise_out,timee_out]=Inverse_approach(hol(FOV_size1,FOV_size2),mrate,pix,lambda,ranges,0,REMOVED_I0,sigma,Mask,Alpha_thresh,K,itter,factors(3),factors(1:2),SecondT,UNIF_Z,MEXQmap,MEXmodelFAST,NoFigure);
                       precise=[precise;precise_out];timee=[timee;timee_out];
                        flag_glob=-1;
                        break;
                end


            elseif ((alpha1/nom< Alpha_thresh) && FOV==0)
                     if (iI==1)
                        fprintf('1. your provided range for Z and R \n 2. threshold for alpha \n 3. Can you see any particles on hologram? Remove the background from Raw holograms using a sequence of captured holograms\n')
                     found=0;precise=0;timee=0;

                     else
                  
                    fprintf('- no more particles inside the field of view. Results are ready\n')
                    
                     end
                        FOV=1;
                        flag_glob=-1;
                        break;
            else


     %% local optimization  

     
        precises=founds;

 

if N==1
    t1=toc;
    tic
end
            [dx,dy,dz,dr]=CRLB_point(precises(1),precises(2),precises(3),precises(4),pix,lambda,MS,MR,sigma/N,N,A0); 
        if (dx>0 && dz>0 && dr>0)
            factorq=[dx dz dr factors(3) itter];
        else
            factorq=[1e-9 2e-11 5e-10 coef itter];
        end  

            fprintf('- local optimization\n')
            precises=local_optim_ds(hologram_ds,precises,lambda,pix,wds,FOV_size1N,FOV_size2N,N,tt,factorq,ranges,MEXmodelFAST);  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%her
          
if N==1
    t2=toc;
end           
            
        while(floor(N/2)>1)
            clear hologram_ds
            
            N=floor(N/2);
            fprintf('- Down-sampling period in one dimension is %g\n',N)

            tt=N;
%             pixel=pix*N;
            fil=zeros(pad_size(1),pad_size(2));
            fil(floor(pad_size(1)/2-N/2)+1:floor(pad_size(1)/2+N/2),floor(pad_size(2)/2-N/2)+1:floor(pad_size(2)/2+N/2))=1/N/N*ones(N);
            hologram_ds=ifftshift(ifft2(fft2(hol).*fft2(fil)));   

            % % Down sampling
            hologram_ds=hologram_ds(1:N:end,1:N:end);
            hologram_ds=hologram_ds(1:2*floor(size(hologram_ds,1)/2),1:2*floor(size(hologram_ds,2)/2));
            wds=w(1:N:end,1:N:end);
            wds=wds(1:2*floor(size(hologram_ds,1)/2),1:2*floor(size(hologram_ds,2)/2));
            map_FOV1N=floor(size(hologram_ds,1)/(mrate(1)+1)/2)+1:floor(size(hologram_ds,1)-size(hologram_ds,1)/(mrate(1)+1)/2);
            map_FOV2N=floor(size(hologram_ds,2)/(mrate(2)+1)/2)+1:floor(size(hologram_ds,2)-size(hologram_ds,2)/(mrate(2)+1)/2);

            FOV_size1N=floor(size(hologram_ds,1)/2)-floor(size(hologram_ds,1)/2/(mrate(1)+1))+1:floor(size(hologram_ds,1)/2)+floor(size(hologram_ds,1)/2/(mrate(1)+1));
            FOV_size2N=floor(size(hologram_ds,2)/2)-floor(size(hologram_ds,2)/2/(mrate(2)+1))+1:floor(size(hologram_ds,2)/2)+floor(size(hologram_ds,2)/2/(mrate(2)+1));
                
            Ids=sum(sum(hologram_ds.*wds))/sum(sum(wds(FOV_size1N,FOV_size2N)));
            hologram_ds(FOV_size1N,FOV_size2N)=hologram_ds(FOV_size1N,FOV_size2N)-Ids; 

            
            [dx,dy,dz,dr]=CRLB_point(precises(1),precises(2),precises(3),precises(4),pix,lambda,MS,MR,sigma/N,N,A0);

        if (dx>0 && dz>0 && dr>0)
            factorq=[dx dz dr factors(3) itter];
        else
            factorq=[1e-9 2e-11 5e-10 coef itter];
        end  
            precises=local_optim_ds(hologram_ds,precises,lambda,pix,wds,FOV_size1N,FOV_size2N,N,tt,factorq,ranges,MEXmodelFAST); %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%her factor dx

        
        end


    if N>1
        t1=toc;

        N=1;
        fprintf('- Down-sampling period in one dimension is %g\n',N)

        tt=K;
            mm=sum(sum(hol(FOV_size1,FOV_size2).*w(FOV_size1,FOV_size2)))/sum(sum(w(FOV_size1,FOV_size2)));
            hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2)-mm;

        tic
            [dx,dy,dz,dr]=CRLB_point(precises(1),precises(2),precises(3),precises(4),pix,lambda,MS,MR,sigma,K,A0);
        if (dx>0 && dz>0 && dr>0)
            factorq=[dx dz dr factors(3) itter];
        else
            factorq=[1e-9 2e-11 5e-10 coef itter];
        end  
            precises=local_optim_ds(hol,precises,lambda,pix,w,FOV_size1,FOV_size2,1,K,factorq,ranges,MEXmodelFAST); %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%here!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        t2=toc;
            



    end 
    
    
        if (((abs(precises(1)/pix) > holsize(1)/2) || (abs(precises(2)/pix) > holsize(2)/2)) && (FOV==1))
            fprintf('- This particle is placed out of the field of View of the Camera\n')
            FOV=0;
            factorQ=factorQ/2;
%             w_prim(FOV_size1,FOV_size2)=0;
        end
        %% check alpha to see if we are out of particles after optimization
            mm=sum(sum(hol(FOV_size1,FOV_size2).*w(FOV_size1,FOV_size2)))/sum(sum(w(FOV_size1,FOV_size2)));
            hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2)-mm;
            if (SecondT==1 || MEXmodelFAST~=1)
                [g_current dumy jini]=model_mex(precises(1),precises(2),precises(3),precises(4),pix,lambda,size(hol),0,K);
                clear dumy
            else
                g_current=model_calc_faster(precises(1),precises(2),precises(3),precises(4),pix,lambda,size(hol),0,K);
            end
        
        wg=sum(sum(w.*g_current)); % wg
        wrg=sum(sum(w.*hol.*g_current)); %wrg
        wg2=sum(sum(w.*(g_current.^2)));%wg2
        ws=sum(sum(w));
        wr=sum(sum(w.*hol));

                alpha=ws.*(wg.*wr-wrg)/(ws.*wg2 - wg.^2);
                I_p=(wg.^2.*wr-wg.*wrg)./(ws.*wg2 - wg.^2);
         nom=I_p+I0+mm;

           fprintf('- Value of alpha which should be thresholded is %g\n',alpha/nom) % value of alpha

        

 timee(iI,:)=[t1 t2];
         if alpha/nom< Alpha_thresh
          
        if (FOV==1 && (mrate(1)>1 || mrate(2)>1))
             fprintf('- Searching out of the filed of view using one-scale approach\n')
                 factors(1:2)=factors(1:2)/2;
                 FOV=0;
                itter=30;
                [hol1,found,precise_out,timee_out]=Inverse_approach(hol(FOV_size1,FOV_size2),mrate,pix,lambda,ranges,FOV,REMOVED_I0,sigma,Mask,Alpha_thresh,K,itter,factors(3),factors(1:2),SecondT,UNIF_Z,MEXQmap,MEXmodelFAST,NoFigure);
                precise=[precise;precise_out];timee=[timee;timee_out];
        else
                precise(iI,1:4)=founds;
        end
                flag_glob=-1;
                break;
         end 


 %% cleaning using formula no.8 in [1] - using second order term
      I0=I0+I_p+mm;
     if (SecondT==1 && MEXmodelFAST~=1)

         hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2) - ( I_p - alpha.* g_current(FOV_size1,FOV_size2)+ I0* ((alpha/2/I0).^2).* ((precises(4)^2*pi/lambda/precises(3)).^2) * (jini(FOV_size1,FOV_size2).^2)) ;
         clear g_current
         clear jini
     else
         
         hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2) - ( I_p - alpha.* g_current(FOV_size1,FOV_size2));
         clear g_current
     end
         
         if NoFigure~=1
            figure;imshow(hol(FOV_size1,FOV_size2),[]);title(sprintf('The residual hologram after the cleaning of particle number %g with FAST',iI));axis on;colorbar
        end

                precises(1:2)=precises(1:2)/pix; 
                 found(iI,:)=founds(:); % coordinates of the found particle by matching pursuit in pixels
                 precise(iI,1:4)=precises(1:4);% coordinates of the final found particle in pixels
                 precise(iI,5)=alpha; 
                 precise(iI,6)=I_p;
                 precise(iI,7)=I0;
                 precise(iI,8)=Level; % level of the pyramid where it has found the particle
                 if precise(iI,3)<ranges(1,1) 
                     fprintf('- Z is smaller than the lower bound of range\n')
                 elseif precise(iI,3)>ranges(1,2) 
                     fprintf('- Z is bigger than the upper bound of range\n')
                 elseif precise(iI,4)<ranges(2,1) 
                    fprintf('- R is smaller than the lower bound of range\n')
                 elseif precise(iI,4)>ranges(2,2)
                     fprintf('- R is bigger than the upper bound of range\n')
                 end




                 if exist('FAST_found.mat','file')    
                    save FAST_found.mat found -append
                    clear found(iI-1,:,ii)
                 else 
                     save FAST_found.mat found
                 end

                 if exist('FAST_times.mat','file')    
                    save FAST_times.mat timee -append
                    clear timee(iI-1,:)
                 else 
                     save FAST_times.mat timee
                 end

                 if exist('FAST_precise.mat','file')    
                    save FAST_precise.mat precise -append
                    clear precise(iI-1,:)
                 else 
                     save FAST_precise.mat precise
                 end


              iI=iI+1;
              fprintf('- particle number:%g\n',iI)
              flag_loc=-1;
              continue;
                 
            end
    end
    
    

end

                    % save and overwrite time information
                     save FAST_times.mat timee
                    % save and overwrite particle information
                     save FAST_precise.mat precise
    %% show the cleaned hologram

     figure;imshow(hol(:,:,1),[]);title('completely cleaned hologram with FAST- In the field detection');


