% /*
% %-------------------------------------------------------------------------%
% % Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
% %                               http://laboratoirehubertcurien.fr/
% % 
% % Contact : corinne.fournier@univ_st_etienne.fr
% % Sept 2011
% %-------------------------------------------------------------------------% 
%  * Qmap_mex.cpp - Likelihood map between provided hologram and simulated diffraction patterns of a particle- Formula 22 in  [1].
%  *
%  * ======================================================================
%  * Usage: [row,col,maxim,q0]=Qmap_mex(hol,lambda,pix,w,r,z,w_prim,MapFOVx,MapFOVy,K)
%  * ======================================================================
%  * inputs:
%  * hol: zero-padded hologram 
%  * sizeX: Width of the hologram in pixels
%  * sizeY: Length of the hologram in pixels
%  * lambda : wavelenght of laser in meter
%  * pix: pixel size of sensor in meter
%  * w: weigthing matrix ccontaining mask or information about dead pixels of sensor
%  * r: Radius of particle in meter
%  * z: axial coordinate of particle in meter
%  * w_prim: weigthing matrix for out-of Filed detection
%  * MapFOVx: 1-by-2 Matrix containing range of vertical-indexes of holo which containes original hologram
%  * MapFOVy: 1-by-2 Matrix containing range of horizontal-indexes of holo which containes original hologram
%  * K: fill factor
%  *
%  * ======================================================================
%  * outputs:
%  * row: the centered vertical coordinate of the maximum likelihood pixel 
%  * column: the centered horizontal coordinate of the maximum likelihood pixel
%  * maximi: the maximum of the likelihood map
%  * q0: the sizeX-by-sizeY likelihood map
%  *
%  * ======================================================================
%  * Test:
%  * load hologram_test.mat; hol=zeros(2*size(hologram_test));w=zeros(size(hol));w_prim=ones(size(hol));
%  * hol(513:1536,513:1536)=hologram_test;w(513:1536,513:1536)=1;MapFOVx=[513 1536];k=sqrt(0.7);
%  * [row,col,maxim,q0]=Qmap_mex(hol,0.532e-6,7e-6,w,50e-6,0.12,w_prim,MapFOVx,MapFOVx,k);
%  *=======================================================================
%  * Time on Cetautomatix: 2.5126 seconds - size of hol = [2048 2048]
%  *=======================================================================
%  *[1] F. Soulez, L. Denis, E. Thiebaut, C. Fournier, et C. Goepfert, "Inverse problem approach in particle digital holography:
%  * out-of-field particle detection made possible",Journal of the Optical Society of America A, vol. 24, no. 12, p. 3708-3716, 2007
%  * ======================================================================
%  */
