% /*
% %-------------------------------------------------------------------------%
% % Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
% %                               http://laboratoirehubertcurien.fr/
% % 
% % Contact : corinne.fournier@univ_st_etienne.fr
% % Sept 2011
% %-------------------------------------------------------------------------% 
%  * simulate_holo_mex.cpp - simulated diffraction patterns of some spherical particles on sensor according to formula number 1 in [1].
%  * ======================================================================
%  * Usage: [hologram,particles,Mean,noise]=simulate_holo_mex(n,lambda,pix,sX,sY,I0,SNR,rangeZ,rangeR,random_coordinates,coordinates,meanh,K)
%  * ======================================================================
%  * inputs:
%  * n: number of partciles in the hologram 
%  * lambda : wavelenght of laser in meter
%  * pix: pixel size of sensor in meter
%  * sX: Width of the hologram in pixels
%  * sY: Length of the hologram in pixels
%  * I0: Laser Intensity 
%  * SNR: desired SNR of the produced signal
%  * rangeZ: 1-by-2 matrix containing lower bound and upper bound on the Z-distance value of particles (in meter)
%  * rangeR: 1-by-2 matrix containing lower bound and upper bound on the Radius value of particles (in meter)
%  * random_coordinates: a flag to choose between random production of particle coordinates(=1) , or (=else)use of provided coordinate values present in the next input
%  * coordinates: if the previous input is !=1, containes the coordinates for the particles : a n-by-4 matrix containing [x y z r] of particles all in meters, x and y centered on image
%  * meanh: 0 if a zero-mean hologram is required
%  * K: fill-factor of camera
%  * ======================================================================
%  * outputs:
%  * hologram: sX-by-sY matrix of produced hologram 
%  * particles: coordinates of particles present in the hologram
%  * Mean: the value of mean of hologram if the last input is set to zero.
%  * noise: sX-by-sY matrix contained the noise value of each pixel
%  * ======================================================================
%  * Test:
%  * [hologram_test,particles_test,Mean,noise]=simulate_holo_mex(5,0.532e-6,7e-6,1024,1024,1,10,[0.1 0.2],[50e-6 70e-6],1,0,0,sqrt(0.7));
%  * =======================================================================
%  * Time on Cetautomatix: 1.95 seconds : size of hol = [1024 1024] with 1 particle
%  * Time on Cetautomatix: 2.9304 seconds : size of hol = [1024 1024] with 5 particles
%  * ======================================================================
%  *[1] F. Soulez, L. Denis, E. Thiebaut, C. Fournier, et C. Goepfert, "Inverse problem approach in particle digital holography:
%  * out-of-field particle detection made possible", Journal of the Optical Society of America A, vol. 24, no. 12, p. 3708-3716, 2007
%  * ======================================================================
%  */