function [hol,found,precise,timee]=Inverse_approach(hologram,mrate,pix,lambda,ranges,FOV,REMOVED_I0,SIGMA,Mask,Alpha_thresh,K,itter,coef,factorQ,SecondT,UNIF_Z,MEXQmap,MEXmodelFAST,NoFigure)
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Goal : process of a Digital Hologram to extract the 3D coordinates and
% the Radius of spherical particle using Inverse Approach
%
%-------------------------------------------------------------------------%
% inputs:         
% hologram
%
% mrate         = the rate of the searching widnow comparing to the
%                hologram's size - no unit
%
% pix           = pixel size in meter
%
% lambda        = wavelenght of the laser with which the hologram has
%               been recorded
%
% ranges        = ranges for Radius and Z distance of particles,
%               known from the experiment and the setup
%
% FOV           = 0 if one searches out of the filed of view od sensor for particles
% REMOVED_I0    = if you have removed the mean of hologram please provide
%               the value of mean. Otherwise pasthis parameter as 0  - I0
%               couldn't be negative!
% SIGMA         = the standard deviation of uncorrelated gaussian noise  
% Mask          = wieghting mask over considered pixels of sensor
%
% Alpha_thresh  = threshold on value of alpha(=0 generally)
% K             = fill factor of camera
% itter         = number of iterations of optimization 
% coef          = coef*CRLB is the lower limit on the accuracy of optimization
% factorQ       = [Z_step R_step]; sampling step size in matching pursuit step
% SecondT       = 1 if second term of EQ. should be counted in
% UNIF_Z        = 1 if unifrom sampling of Z space desired in matching pursuit
% MEXQmap       = 1 if the mex version of Qmap in matching pursuit is required , 0 oherwise
% MEXmodelFAST  = 1 if model with multi-threading is required
% NoFigure      = 1 if it is not necessary to plot the intermediate holograms 
%-------------------------------------------------------------------------%
%outputs        
% found         = coordinates of found particles during global detection 
% precise       = coordinates of all the particles which have been
%               detected from the hologram after local optimization - final values
%
% timee         =nby2 matrix, with n equal to number of found particles,first column corresponds to time cost of matching pursuit step, second column corresponds to time cost of local optimization

%-------------------------------------------------------------------------%

% To test:
% lambda=0.532e-6;
% pix=7e-6;
% hol_size=[1024 1024];
% I0=1;
% SNR=100;
% ranges=[0.1 0.2;50e-6 70e-6];
% mrate=[1 1];
% n=1; %number of particles in the hologram
% FOV=1;
% K=sqrt(0.7);
% factorQ=[0.002 2e-6];Alpha_thresh=1;
% Mask = 0; itter=20; SecondT=1; UNIF_Z=1; coef=10; MEX_Q=0;
% [hologram,particle,REMOVED_I0,noises]=simulate_holo_mex(n,lambda,pix,hol_size(1),hol_size(2),I0,SNR,ranges(1,:),ranges(2,:),1,0,0,K);
% SIGMA=I0*pi*(max(particle(:,4)).^2)/lambda/min(particle(:,3))/SNR; 
% figure;imshow(hologram,[]); 
% [hol,found,precise,timee]=Inverse_approach(hologram,mrate,pix,lambda,ranges,FOV,REMOVED_I0,SIGMA,Mask,Alpha_thresh,K,itter,coef,factorQ,SecondT,UNIF_Z,MEX_Q,1,1);
% 
%!-------------------------------------------------------------------------%
%! processing time : ~664.819 s on Cetautomatix .  
%! profiler : Inverse problem=0.741s, Qmap_mex=565.569s(218 calls) ,model_mex=52.006 s(78 calls)
%! local_optim_difference=11.502 s,matching pursuit=0.059 s,
%! Gradient_filtering=1.067s, sinc=0.613s(5 calls) , j0c=0.227s, j1c=0.227s ...
%-------------------------------------------------------------------------%
% functions used : matching_pursuit.m , CRLB_point.m , local_optim.m , global_detection.m , Gradients_filtering.m , Qmap_mex.cpp , model_mex.cpp, j1c.cpp, j0c.cpp
%-------------------------------------------------------------------------%
% [1] F. Soulez, L. Denis, E. Thiebaut, C. Fournier, et C. Goepfert, "Inverse problem approach in particle digital holography:
%  out-of-field particle detection made possible", Journal of the Optical Society of America A, vol. 24, no. 12, p. 3708-3716, 2007
%-------------------------------------------------------------------------%

fprintf(' - This process can take some minutes per particle \n depending on the size of your search space.\n \n - It took us 10 minuets per particle for a volume of \n 7 by 7 by 100 millimeters searching for particles \n with radii between 50 to 70 micrometers.\n \n - Until the process is complete, be our guest and take a Coffee break!\n')
load path_data.mat

%% Zero-padding

holsize=size(hologram);
pad_size(1)=(mrate(1)+1)*holsize(1);
pad_size(2)=(mrate(2)+1)*holsize(2);
%% calculation of map indices 
% % Filed of search - indexing according to Matlab
map_FOV1=holsize(1)/2+1:pad_size(1)-holsize(1)/2;
map_FOV2=holsize(2)/2+1:pad_size(2)-holsize(2)/2;

% % to use mex function of matching pursuit - indexing according to c language
MFOVX(1)=floor(holsize(1)/2);
MFOVX(2)=pad_size(1)-floor(holsize(1)/2);
MFOVY(1)=floor(holsize(2)/2);
MFOVY(2)=pad_size(2)-floor(holsize(2)/2);


%% calculation of Field-of-view indices
FOV_size1=(pad_size(1)-holsize(1))/2+1 :(pad_size(1)+holsize(1))/2 ;
FOV_size2=(pad_size(2)-holsize(2))/2+1 :(pad_size(2)+holsize(2))/2 ;

%% initialization
hol=zeros(pad_size(1),pad_size(2));
hol(FOV_size1,FOV_size2)=hologram;

%% finding the best fft algorithm
    fftw('wisdom', []);
    fftw('planner', 'exhaustive');
    dummy=fft2(hol);
    clear dummy
    

%% Mask set and calculation of I0  
    if (exist('Mask','var') && (size(Mask,1)==size(hologram,1)) && (size(Mask,2)==size(hologram,2)))
        w=zeros(pad_size(1),pad_size(2));
        w(FOV_size1,FOV_size2)=Mask;
        ind=find(Mask~=0);


        if REMOVED_I0==0
            I0=mean(mean(hologram(ind)));
            hol1(ind)=hologram(ind)-I0;
        else
            I0=REMOVED_I0;
            hol1=hologram;
        end
        %% SNR
        hh=sort(hol1(ind));
        %SIGMA=std(hh(floor(size(hh,1)/2)-25:floor(size(hh,1)/2)+25));
        A_SIGNAL=mean(hh(size(hh,1)-50:size(hh,1)));
        figure;imshow(hol1.*Mask,[]);title('after removing mean using Mask')
    else
        fprintf('- there is no Mask or the size of Mask is not equal to the field of view of the sensor, Mask is set to ones all over the FOV of sensor\n')
        w=zeros(pad_size(1),pad_size(2));
        w(FOV_size1,FOV_size2)=1;

        if REMOVED_I0==0
            I0=mean(mean(hologram));
            hol1=hologram-I0;
        else 
            I0=REMOVED_I0;
            hol1=hologram;
        end
        %% SNR
        hh=sort(hol1(:));
        %SIGMA=std(hh(floor(size(hh,1)/2)-25:floor(size(hh,1)/2)+25));
        A_SIGNAL=mean(hh(size(hh,1)-50:size(hh,1)));
    end

    hol(FOV_size1,FOV_size2)=hol1;
    clear hol1
    


%% initializing
% iteration counter
iI=1;


% to account for out of field particles
% w_prim=ones(size(map_FOV1,2),size(map_FOV2,2));
 w_prim=ones(size(hol));
%% main loop of finding particles
    while(true)

       
        fprintf('- particle number:%g\n',iI)
       
        % to account for out of field particles
        if FOV==0
           w_prim(FOV_size1,FOV_size2)=0;
        end
   
    %removing the mean of hologram in the field of view
        mm=sum(sum(hol(FOV_size1,FOV_size2).*w(FOV_size1,FOV_size2)))/sum(sum(w(FOV_size1,FOV_size2)));
        hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2)-mm;
        %% global detection 
        fprintf('- Matching Pursuit procedure\n') 
        
        fprintf('- Field of View is %g\n',FOV)
        
        fprintf('=1 to search in the Field of view \n =0 to search out of the Field of view\n')
        tic
        % Matching Pursuit procedure
        
       
       
        
%             % matching pursuit using .m file for Qmap
            founds=matching_pursuit(hol,lambda,pix,holsize,FOV,w,ranges,map_FOV1,map_FOV2,0,K,factorQ,UNIF_Z,MEXQmap,MEXmodelFAST);
            
%             % matching pursuit using .mex file for Qmap
%         founds=matching_pursuit(hol,lambda,pix,holsize,FOV,w,ranges,MFOVX,MFOVY,0,K,factorQ,UNIF_Z,MEX);


%%
                                    
%             % matching pursuit with Non-Uniform sampling using Rayleigh
%             % resolution formulas using .mex file
%         founds=matching_pursuit_NUS(hol,K,lambda,pix,mean(holsize),w,w_prim,ranges(1,:),ranges(2,:),MFOVX,MFOVY);
        
%             % matching pursuit with Uniform sampling with sample step size
%             % provided as factorQ using .mex file
%         founds=matching_pursuit_US(hol,lambda,pix,w,w_prim,ranges(1,:),ranges(2,:),MFOVX,MFOVY,K,factorQ);
        t1=toc;

        %% check alpha to see if we are out of particles after matching pursuit
if (MEXmodelFAST==1)
     g_current=model_calc_faster(founds(1)*pix,founds(2)*pix,founds(3),founds(4),pix,lambda,pad_size,0,K);
        
%         figure;imshow(g_current,[]);title(sprintf('model %g',iI));
else
       
        g_current=model_mex(founds(1)*pix,founds(2)*pix,founds(3),founds(4),pix,lambda,pad_size,0,K);
end
        wg=sum(sum(w.*g_current)); % wg
        wrg=sum(sum(w.*hol.*g_current)); %wrg
        wg2=sum(sum(w.*(g_current.^2)));%wg2
        ws=sum(sum(w));
        wr=sum(sum(w.*hol));

            alpha1=ws.*(wg.*wr-wrg)/(ws.*wg2 - wg.^2);
            I_p1=(wg.^2.*wr-wg.*wrg)./(ws.*wg2 - wg.^2);

        nom=I_p1+I0+mm;
        fprintf('- Value of alpha which should be thresholded is %g \n',alpha1/nom) % value of alpha
        
        % if alpha is smaller than threshold for the first particle
        if (((alpha1/nom < Alpha_thresh) && (iI==1)) && (FOV==1))
            fprintf('- after global detection no first particle in the field!\n')
                if mrate(1)==1 && mrate(2)==1
                fprintf('- please check :\n 1. your provided range for Z and R \n 2. threshold for alpha \n 3. Can you see any particles on hologram? Remove the background from Raw holograms using a sequence of captured holograms\n')
                    FOV=1;
                    
                    found=zeros(1,4);precise=zeros(1,8);timee=zeros(1,2);
                    break;
                
                else % go to out-of-field for searching
                    fprintf('- Searching out of the field of view\n')
                    factorQ=factorQ/2;
                     FOV=0;
                     
                    continue;
                end
        elseif (((alpha1/nom < Alpha_thresh) && (iI==1)) && (FOV==0))
            fprintf('- after global detection no first particle out of the field! \n please check :\n ')
                fprintf('1. your provided range for Z and R \n 2. threshold for alpha \n 3. Can you see any particles on hologram? Remove the background from Raw holograms using a sequence of captured holograms\n')

                found=zeros(1,4);precise=zeros(1,8);timee=zeros(1,2);
                    break;
        elseif ((alpha1/nom < Alpha_thresh) && (FOV==1))
                fprintf('- no more particles inside the field of view\n')

                if mrate(1)==1 && mrate(2)==1
                   fprintf('- Results are ready\n')
                    FOV=1;
                    
                    break;
                else 
                    fprintf('- Searching out of the filed of view\n')
                    factorQ=factorQ/2;
                     FOV=0;
                     save holo_LMFA_intheF_cleaned.mat hol
                    continue;
                end

        elseif ((alpha1/nom< Alpha_thresh) && FOV==0)

          fprintf('- No more particles could be found out of the field. Results are ready\n')
                    FOV=1;
                    
                    break;
        else
        if (((abs(founds(1)) > holsize(1)/2) || (abs(founds(2)) > holsize(2)/2)) && (FOV==1))
            fprintf('- This particle is placed out of the field of View of the Camera\n')
            FOV=0;
            factorQ=factorQ/2;
%             w_prim(FOV_size1,FOV_size2)=0;
        end

     %% local optimization   

        fprintf('- local optimization\n')

        founds(1:2)=founds(1:2)*pix;
        
        
        [dx,dy,dz,dr]=CRLB_point(founds(1),founds(2),founds(3),founds(4),pix,lambda,holsize,mrate,SIGMA,K,A_SIGNAL);
        if (dx>0 && dz>0 && dr>0)
            factors=[dx dz dr coef itter];
        else
            factors=[1e-9 2e-11 5e-10 coef itter];
        end    
        tic                
        precises=local_optim(hol,founds,lambda,pix,w,FOV_size1,FOV_size2,I0,K,factors,ranges,SecondT,MEXmodelFAST);
        t2=toc;
        %% output time is updated
        timee(iI,:)=[t1 t2];       
        precises(1:2)=precises(1:2)/pix;
        
        %% check alpha to see if we are out of particles after optimization
            if (SecondT==1) || (MEXmodelFAST~=1)
                [g_current dumy jini]=model_mex(precises(1)*pix,precises(2)*pix,precises(3),precises(4),pix,lambda,pad_size,0,K);
                clear dumy
            else
                g_current=model_calc_faster(precises(1)*pix,precises(2)*pix,precises(3),precises(4),pix,lambda,pad_size,0,K);
            end
        
        clear dumy
        wg=sum(sum(w.*g_current)); % wg
        wrg=sum(sum(w.*hol.*g_current)); %wrg
        wg2=sum(sum(w.*(g_current.^2)));%wg2
        ws=sum(sum(w));
        wr=sum(sum(w.*hol));

                alpha=ws.*(wg.*wr-wrg)/(ws.*wg2 - wg.^2);
                I_p=(wg.^2.*wr-wg.*wrg)./(ws.*wg2 - wg.^2);
         nom=I_p+I0+mm;
         fprintf('- Value of alpha which should be thresholded is %g\n',alpha/nom) % value of alpha

        
         

         
         % if alpha is lower than threshold considreing that after Matching pursuit we found a reasonable alpha
         if alpha/nom< Alpha_thresh
                fprintf('- oooops, local optimization found negative alpha in contrary to matching pursuit. Check the results for ghost particles or missed particles\n')  
                if FOV==1
                    factorQ=factorQ/2;
                     FOV=0;
                    continue;
                else
                    precise(iI,1:4)=founds;
                    break;
                end
         end 








     %% cleaning using formula no.8 in [1] - using second order term
     I0=I0+I_p+mm;
     if (SecondT==1 && MEXmodelFAST~=1)

         hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2) - ( I_p - alpha.* g_current(FOV_size1,FOV_size2)+ I0* ((alpha/2/I0).^2).* ((precises(4)^2*pi/lambda/precises(3)).^2) * (jini(FOV_size1,FOV_size2).^2)) ;
         clear jini
     else
         
         hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2) - ( I_p - alpha.* g_current(FOV_size1,FOV_size2));
     end
        clear g_current
        
        if NoFigure~=1
            figure;imshow(hol(FOV_size1,FOV_size2),[]);title(sprintf('The residual hologram after the cleaning of particle number %g',iI));axis on;colorbar
        end
        %% outputs are updated
                 found(iI,:)=founds(:);
                 precise(iI,1:4)=precises(1:4);
                 precise(iI,5)=alpha;
                 precise(iI,6)=I_p;
                 precise(iI,7)=I0;
                 precise(iI,8)=1;
                 
                 % check if some parameters are out of range
                 if precise(iI,3)<ranges(1,1) 
                     fprintf('- Z is smaller than the lower bound of range\n')
                 elseif precise(iI,3)>ranges(1,2) 
                     fprintf('- Z is bigger than the upper bound of range\n')
                 elseif precise(iI,4)<ranges(2,1) 
                    fprintf('- R is smaller than the lower bound of range\n')
                 elseif precise(iI,4)>ranges(2,2)
                     fprintf('- R is bigger than the upper bound of range\n')
                 end

                 if exist(strcat(path_data,'times_toolbox.mat'),'file') 
                    save(strcat(path_data,'times_toolbox.mat'),'timee','-append');
%                     save times_toolbox.mat timee -append
                    clear timee(iI-1,:)
                 else 
                     save(strcat(path_data,'times_toolbox.mat'),'timee');
%                      save times_toolbox.mat timee
                 end

                 if  exist(strcat(path_data,'precise_toolbox.mat'),'file') 
                    save(strcat(path_data,'precise_toolbox.mat'),'precise','-append');
%                     save precise_toolbox.mat precise -append
                    clear precise(iI-1,:)
                 else 
                    save(strcat(path_data,'precise_toolbox.mat'),'precise');
%                      save precise_toolbox.mat precise
                 end

                 %% change itteration to a new particle detection
                 iI=iI+1;
                 
        end

    end
    %% show the cleaned hologram
    hol=hol(FOV_size1,FOV_size2);
    figure;imshow(hol,[]);title('Hologram : completely cleaned - One-scale approach');axis on;colorbar
    
    
