function found=matching_pursuit(hol,lambda,pix,holsize,FOV,w1,ranges,MapFOVx,MapFOVy,N,tt,factors,UNIF_Z,MEXQmap,MEXmodelFAST)%
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Goal : Matching-pursuit search (global detection) in 4D search space for
% spherical particles
%
%-------------------------------------------------------------------------%
%
% inputs:
%
% hol                    = hologram
% lambda                 = wavelength in meter
% pix                    = pixel size in meter
% holsize                = size of the original hologram without extended window of search in pixel
% FOV                    = 1 to serach for the in-the-field-of-view particles, 0 for out of field detection
% w1                     = wighting matrix or Mask
% ranges                 = range of z and r. a 2-by-2 matrix in meter : [z_lb z_ub; r_lb r_ub]
% MapFOVx                = 1-by-2 Matrix containing range of vertical-indexes of holo which containes original hologram
%                          [x_first:x_last] if arg #14 MEXQmap=0 ; 
%                          [x_first x_last] if MEXQmap=1 ;
% MapFOVy                = 1-by-2 Matrix containing range of horizontal-indexes of holo which containes original hologram
%                          [y_first:y_last] if arg #14 MEXQmap=0 ; 
%                          [y_first y_last] if MEXQmap=1 ;
% N                     = (downsampling) ? (downsampling rate in pixels : 1) 
% tt                    = (downsampling) ? (downsampling rate in pixels : fill-factor) 
% factors               = [Z_step R_step]; sampling step size in matching pursuit step
% UNIF_Z                = 1 if unifrom sampling of Z is required
% MEXQmap               = 1 if mex function of Qmap should be used
% MEXmodelFAST          = 1 if model with multi-threading is required
%-------------------------------------------------------------------------%
%
% outputs:
%
% found             = x(in pixels),y(in pixels),z(in meters),r(in meters) coordinates of the particle with highest peak in Qmap
%-------------------------------------------------------------------------%
% 
% To test:
% n=5; lambda=0.532e-6; pix=7e-6; holsize=[1024 1024]; I00= 5;
% SNR=100; FOV=1;tt=sqrt(0.7);UNIF_Z=1; N=0;
% ranget=[0.2 0.25;50e-6 60e-6]; 
% meanh=0; factors=[0.002 2e-6];coordinates=0;
% [hol1,particle,Mmean,noise]=simulate_holo_mex(n,lambda,pix,holsize(1),holsize(2),I00,SNR,ranget(1,:),ranget(2,:),1,coordinates,meanh,tt);
% holo=zeros(2*holsize);
% holo(holsize(1)/2+1:3*holsize(1)/2,holsize(2)/2+1:3*holsize(2)/2)=hol1;
% w1=zeros(2*holsize);w1(floor(holsize(1)/2)+1:3*floor(holsize(1)/2),floor(holsize(2)/2)+1:3*floor(holsize(2)/2))=1; MapFOVx=[floor(holsize(1)/2)+1:3*floor(holsize(1)/2)];MapFOVy=[floor(holsize(2)/2)+1:3*floor(holsize(2)/2)];
% found=matching_pursuit(holo,lambda,pix,holsize,FOV,w1,ranget,MapFOVx,MapFOVy,N,tt,factors,UNIF_Z,0,1);
%!-------------------------------------------------------------------------%
%! processing time : ~229.452 s on Cetautomatix .  
%! profiler : matching pursuit=0.04 s, Qmap_mex=227.205s(109 calls), global detection= 0.01s(3 calls)
%-------------------------------------------------------------------------%
% functions used : global_detection.m , Qmap_mex.cpp , j1c.cpp, j0c.cpp
%-------------------------------------------------------------------------%
% [1] F. Soulez, L. Denis, E. Thiebaut, C. Fournier, et C. Goepfert, "Inverse problem approach in particle digital holography:
%  out-of-field particle detection made possible", Journal of the Optical Society of America A, vol. 24, no. 12, p. 3708-3716, 2007
%-------------------------------------------------------------------------%


% 
w_prim=ones(size(hol));
if FOV==0
    w_prim(floor(size(hol,1)/2-holsize(1)/2)+1:floor(size(hol,1)/2+holsize(1)/2),floor(size(hol,2)/2-holsize(2)/2)+1:floor(size(hol,2)/2+holsize(2)/2))=0;
end
r_begin=ranges(2,1)+(ranges(2,2)-ranges(2,1))/2;
    
    [found_x,found_y,estimated_z,p_max,NOM_R]=global_detection(hol,w1,lambda,pix,1,r_begin,ranges(1,:),holsize(1),w_prim,MapFOVx,MapFOVy,tt,N,factors,UNIF_Z,MEXQmap,MEXmodelFAST); % for Z 
    [found_x1,found_y1,estimated_r,p_max,NOM_R]=global_detection(hol,w1,lambda,pix,2,estimated_z,ranges(2,:),holsize(1),w_prim,MapFOVx,MapFOVy,tt,N,factors,UNIF_Z,MEXQmap,MEXmodelFAST); % for R
    if NOM_R>1
        fprintf('- More than one R-sample\n')
        [found_x,found_y,estimated_z,p_max,NOM_R]=global_detection(hol,w1,lambda,pix,1,estimated_r,ranges(1,:),holsize(1),w_prim,MapFOVx,MapFOVy,tt,N,factors,UNIF_Z,MEXQmap,MEXmodelFAST);% for Z
    end
    
    if (MEXQmap==1)
        found_x=found_x-floor((MapFOVx(2)-MapFOVx(1))/2)-2;
        found_y=found_y-floor((MapFOVy(2)-MapFOVy(1))/2)-2;
    else
        found_x=found_x-size(MapFOVx,2)/2-1;
        found_y=found_y-size(MapFOVy,2)/2-1;
    end
    

    found(1,1)=found_x;
    found(1,2)=found_y;
    found(1,3)=estimated_z;
    found(1,4)=estimated_r;
    
   
end



